import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";

/* eslint handle-callback-err: "off" */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import AbTestToggle from '@s-ui/abtesting-toggle';
import ExperimentContext from './experiment-context';
import OptimizelyX from './optimizely-x';

var AbTestOptimizelyXExperiment =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(AbTestOptimizelyXExperiment, _Component);

  function AbTestOptimizelyXExperiment(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _this._buildContextState = function (_ref) {
      var variationId = _ref.variationId,
          _ref$active = _ref.active,
          active = _ref$active === void 0 ? false : _ref$active;

      var _this$props = _this.props,
          children = _this$props.children,
          propsWithoutChildren = _objectWithoutProperties(_this$props, ["children"]); // remove children prop


      var variationName = _this._getVariationNameFromVariationId(variationId);

      return _objectSpread({}, propsWithoutChildren, {
        isActive: active,
        isDefault: variationId === _this.defaultVariation,
        isVariation: variationId !== _this.defaultVariation,
        isWrapped: true,
        variationId: variationId,
        variationName: variationName
      }, _this._getVariationFlags(variationId));
    };

    _this._getDefaultVariation = function () {
      var defaultChild = _this.props.children.find(function (child) {
        return child.props.defaultVariation;
      });

      return defaultChild ? defaultChild.props.variationId : null;
    };

    _this._getForceVariationId = function () {
      var _this$props2 = _this.props,
          children = _this$props2.children,
          forceVariation = _this$props2.forceVariation;
      var forceVariationChild = forceVariation && children.find(function (child) {
        var variationNameForChild = _this._getVariationNameFromChild(child);

        return (// name or id are both valid ways to force a variation
          forceVariation === variationNameForChild || forceVariation === child.props.variationId
        );
      });
      return forceVariationChild ? forceVariationChild.props.variationId : null;
    };

    _this._getVariationNameFromVariationId = function (variationId) {
      return _this._getVariationNameFromChild(_this.props.children.find(function (child) {
        return child.props.variationId === variationId;
      }));
    };

    _this._getVariationNameFromChild = function (child) {
      // variationName can be overriden via prop
      if (child.props.variationName) return child.props.variationName; // otherwise, use an alphabet letter based on child position

      var childIndex = _this.props.children.indexOf(child);

      return String.fromCharCode(65 + childIndex); // A, B, C, D, ...
    };

    _this._getVariationFlags = function (selectedVariationId) {
      return _this.props.children.reduce(function (obj, child) {
        var _objectSpread2;

        var variationName = _this._getVariationNameFromChild(child); // capitalize variation name in order to get a well-formed camel case key
        // i. e. for a name 'foo' → isVariationFoo


        var capitalizedVariationName = variationName.charAt(0).toUpperCase() + variationName.slice(1);
        return _objectSpread({}, obj, (_objectSpread2 = {}, _objectSpread2["isVariation" + capitalizedVariationName] = child.props.variationId === selectedVariationId, _objectSpread2));
      }, {});
    };

    _this._activationHandler = function (variationId) {
      return _this.setState(_this._buildContextState({
        variationId: variationId,
        active: true
      }));
    };

    _this.defaultVariation = _this._getDefaultVariation();

    var initialVariationId = _this._getForceVariationId() || _this.defaultVariation; // set state which will be used to provide context


    _this.state = _this._buildContextState({
      variationId: initialVariationId
    });
    return _this;
  }

  var _proto = AbTestOptimizelyXExperiment.prototype;

  _proto.componentDidMount = function componentDidMount() {
    if (process.env.NODE_ENV !== 'production' && this.props.forceVariation) {
      var msg = "[OptimizelyXExperiment] Watch out!! Optimizely response will be ignored because the forceVariation prop.";
      if (process.env.NODE_ENV !== 'test') console.warn(msg); // eslint-disable-line no-console

      return;
    }

    OptimizelyX.addActivationListener(this.props.experimentId, this._activationHandler);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    if (process.env.NODE_ENV !== 'production' && this.props.forceVariation) {
      return;
    }

    OptimizelyX.removeActivationListener(this.props.experimentId, this._activationHandler);
  };

  _proto.render = function render() {
    return (
      /**
       * FYI: About why using just `value={this.state}` instead of a new object.
       * @see https://en.reactjs.org/docs/context.html#caveats
       */
      React.createElement(ExperimentContext.Provider, {
        value: this.state
      }, React.createElement(AbTestToggle, {
        variation: this.state.variationId
      }, this.props.children))
    );
  };

  return AbTestOptimizelyXExperiment;
}(Component);

AbTestOptimizelyXExperiment.displayName = 'AbTestOptimizelyXExperiment';
export default AbTestOptimizelyXExperiment;
export { ExperimentContext };